"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.buildHandler = void 0;
const kafkajs_1 = require("kafkajs");
const tx_outbox_1 = require("./tx-outbox");
const env_1 = require("./env");
let producer;
const getProducer = async (brokers, username, password, authMechanism = "plain", useSsl = false) => {
    if (!producer) {
        producer = new kafkajs_1.Kafka({
            clientId: "members-service",
            brokers,
            ssl: useSsl,
            sasl: {
                mechanism: authMechanism,
                username: username,
                password: password,
            },
        }).producer();
        await producer.connect();
    }
    return producer;
};
const buildHandler = ({ brokers, username, password, authMechanism, useSsl }) => (0, tx_outbox_1._handler)(getProducer(brokers, username, password, authMechanism, useSsl));
exports.buildHandler = buildHandler;
// eslint-disable-next-line @typescript-eslint/no-empty-function
exports.handler = process.env.NODE_ENV === "test" ? (() => { }) : (0, exports.buildHandler)({
    brokers: (0, env_1.envOrThrow)("KAFKA_BROKERS").split(","),
    username: (0, env_1.envOrThrow)("KAFKA_USERNAME"),
    password: (0, env_1.envOrThrow)("KAFKA_PASSWORD"),
    authMechanism: (0, env_1.envOrThrow)("KAFKA_AUTH_MECHANISM"),
    useSsl: (0, env_1.envOrThrow)("KAFKA_USE_SSL").toLowerCase() === "true",
});
