"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports._handler = void 0;
const util_dynamodb_1 = require("@aws-sdk/util-dynamodb");
const logging_1 = __importDefault(require("./logging"));
const _handler = (producerP) => async (e) => {
    const batch = e.Records.reduce((batch, next) => {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const { NewImage } = next.dynamodb;
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const object = (0, util_dynamodb_1.unmarshall)(NewImage);
        const { topic, key, value, headers } = object;
        if (!topic || !value) {
            logging_1.default.error("Invalid outbox messages", object);
            throw new Error("Invalid outbox messages: " + JSON.stringify(object));
        }
        const existingBatch = batch[topic] || [];
        const newBatch = {
            ...batch,
            [topic]: [
                ...existingBatch,
                {
                    key: key ? Buffer.from(key, "base64") : undefined,
                    value: Buffer.from(value, "base64"),
                    headers,
                },
            ],
        };
        return newBatch;
    }, {});
    const topicMessages = Object.entries(batch).map(([topic, messages]) => ({
        topic,
        messages,
    }));
    const producer = await producerP;
    await producer.sendBatch({ topicMessages, acks: -1 });
};
exports._handler = _handler;
